# API Documentation of Bash Base Library

## error messages

### error-codes

```
error-codes (bash-base)
#   1 .. unkown internal error
#   2 .. illegal number of arguments in function call
#   3 .. illegal variable name
#   4 .. illegal function arguments
#   5 .. error sourcing file
#   6 .. no such function
#   7 .. variable specified as argument undefined
#   8 .. no such file
#   9 .. no access rights
#  10 .. value not found
#  11 .. incorrect variable type
#  9x .. errors local to each function
```

### bb_error_string

```
bb_error_string <ret-vn> <error-code>
# errors: 2, 3
```

## types

### bb_var_type

```
bb_var_type <ret-basetype-vn> <ret-mod-vn> <var-vn>
# returns the type of a var where basetype can be: str, array or dict
# mod returns the modifiers as returned by declare -p (see bash docu)
# ret-basetype-vn and ret-mod-vn can be empty and then will not be used
# errors: 2, 7
```

### bb_var_is_string

```
bb_var_is_string <vn>
# check if variable is string
# errors: 2, 7, 11
```

### bb_var_is_array

```
bb_var_is_array <vn>
# check if variable is array
# errors: 2, 7, 11
```

### bb_var_is_dict

```
bb_var_is_dict <vn>
# check if variable is dict
# errors: 2, 7, 11
```

### bb_var_assert_string

```
bb_var_assert_string <vn> [<extra-info>]
# checks if vn is valid name (like bb_var_name_validate) and if there is
# a variable of type string with this name, fail if anything is not true
# errors: 2, 3, 7, 11
```

### bb_var_assert_array

```
bb_var_assert_array <vn> [<extra-info>]
# checks if vn is valid name (like bb_var_name_validate) and if there is
# a variable of type array with this name, fail if anything is not true
# errors: 2, 3, 7, 11
```

### bb_var_assert_dict

```
bb_var_assert_dict <vn> [<extra-info>]
# checks if vn is valid name (like bb_var_name_validate) and if there is
# a variable of type dict with this name, fail if anything is not true
# errors: 2, 3, 7, 11
```

### bb_var_assert_callable

```
bb_var_assert_callable <vn> [<extra-info>]
# checks if vn is valid name (like bb_var_name_validate) and if there is
# a variable of type string with this name, fail if anything is not true
# errors: 2, 8
```

### bb_var_configure_want

```
bb_var_configure_want <msg-level>
# configure what bb_var_want_ funs do if variable is undefined:
# if msg-level is specified, output message with level msg-level from now on
# errors: 2, 3
```

### bb_var_want_string

```
bb_var_want_string <vn> [<extra-info>]
# checks if vn is valid name (like bb_var_name_validate), fail if not
# checks if there is a variable of type string with this name, warn if not
# warning can be configured with bb_var_configure_want
# this function is intended bne used for return variables, where it is not
# required to have one defined, but bash-base suggest to do so
# errors: 2, 3, 7, 11
```

### bb_var_want_array

```
bb_var_want_array <vn> [<extra-info>]
# checks if vn is valid name (like bb_var_name_validate), fail if not
# checks if there is a variable of type array with this name, warn if not
# warning can be configured with bb_var_configure_want
# this function is intended bne used for return variables, where it is not
# required to have one defined, but bash-base suggest to do so
# errors: 2, 3, 7, 11
```

### bb_var_want_dict

```
bb_var_want_dict <vn> [<extra-info>]
# checks if vn is valid name (like bb_var_name_validate), fail if not
# checks if there is a variable of type dict with this name, warn if not
# warning can be configured with bb_var_configure_want
# this function is intended bne used for return variables, where it is not
# required to have one defined, but bash-base suggest to do so
# errors: 2, 3, 7, 11
```

## generic tools

### bb_var_name_validate

```
bb_var_name_validate <var-name> [<extra-info>]
# validates variable names
# valid variable names match the following regex: ^[[:alpha:]][[:alnum:]_]*$
# if extra-info is non empty a log msg with level warning will be emitted
# when var-name is invalid
# errors: 2, 3
```

### bb_validate_vn

```
bb_validate_vn <caller-extra-info> <vn> [..]
# validates one or more variable names
# valid variable names match the following regex: ^[[:alpha:]][[:alnum:]_]*$
# if caller-name is non empty a log msg with level warning will be emitted
# THIS FUNCTION IS DEPRECATED
# errors: 2, 3
```

### bb_validate_regex

```
bb_validate_regex <caller-name> <regex> <arg> [..]
# validates one or more arguments against specified regex
# if caller-name is non empty a log msg with level warning will be emitted
# errors: 2, 3
```

### bb_validate_uint

```
bb_validate_uint <caller-name> <uint> [..]
# validates if one or more arguments are unsigned integers
# if caller-name is non empty a log msg with level warning will be emitted
# errors: 2, 3
```

### bb_validate_int

```
bb_validate_int <caller-name> <int> [..]
# validates if one or more arguments are integers
# if caller-name is non empty a log msg with level warning will be emitted
# errors: 2, 3
```

### bb_is_callable

```
bb_is_callable <exec-name>
# checks if exec-name is a callable function or executable in PATH
# errors: 2, 8
```

### bb_executables_in_path

```
bb_executables_in_path <executable-name> [..]
# checks if specified executables present (absolute or relative) or if they
# are in path or they are defined functions
# THIS FUNCTION IS DEPRECATED
# errors: 2, 8
```

### bb_funs_defined

```
bb_funs_defined <caller-name> <function-name> [..]
# checks if specified executables present (absolute or relative) or if they
# are in path or they are defined functions
# errors: 2, 6
```

## string/char tools

### bb_string_compare

```
bb_string_compare <string1> <string2>
# compare if two strings are equal, return 0 if they are
# errors: 2, 91 (not equal)
```

### bb_string_quote

```
bb_string_quote <ret-vn> <string> [<quotation-string>]
# quote a string: ret-vn := <quotation-string><string><quotation-string>
# if no quotation-string is specified, a quotation mark is used
```

### bb_string_trim

```
bb_string_trim <ret-vn> <string> [left|right|both]
# removes whitespaces from string sides, if no side is specified both is used
# errors: 2, 3, 4
```

### bb_string_trim_unicode

```
bb_string_trim_unicode <ret-vn> <string> [left|right|both]
# removes whitespaces from string sides, if no side is specified both is used
# errors: 2, 3, 4
```

### bb_string_trim_custom

```
bb_string_trim_custom <ret-vn> <string> <extra-chars> [left|right|both]
# removes whitespaces from string sides, if no side is specified both is used
# errors: 2, 3, 4
```

### bb_string_split

```
bb_string_split <ret-array-vn> <string> <separator> [<limit>]
# splits string by separator string
# if limit is specified then it splits into at most limit parts starting
# from the left side, limit must be a number > 0
# errors: 2, 3, 4
```

### bb_string_split_regex

```
bb_string_split_regex <ret-array-vn> <string> <regex> [<limit>]
# splits string by regex
# the regex must not contain marked subexpressions and must not match the empty
# string
# if limit is specified then it splits into at most limit parts starting
# from the left side, limit must be a number > 0
# errors: 2, 3, 4, 91 (regex matches empty string)
```

### bb_string_split_regex_full

```
bb_string_split_regex_full <ret-array-vn> <string> <regex> [<limit>]
# splits string by regex, but return delimiters as well as substrings
# the regex must not contain marked subexpressions and must not match the empty
# string
# if limit is specified then it splits into at most limit parts starting
# from the left side, limit must be a number > 0
# note that delimiters to not count as parts
# errors: 2, 3, 4, 91 (regex matches empty string)
```

### bb_char_transliterate

```
bb_char_transliterate <ret-vn> <char> [ascii|username]
# transliterates single char to a ASCII or usernames (ASCII is default)
# when username is used, chars are transliterated to
# ASCII Alphanumeric Chards and the following: _ . $ -
# note: this function returns strings of length 1 or 2
# errors: 2, 3, 4
```

### bb_string_transliterate

```
bb_string_transliterate <ret-vn> <string> [ascii|username]
# transliterates each character of a string with char_transliterate
# optional argument is target charset, ascii is default
# see bb_char_transliterate for more on charsets
# note: the result might be longer than the input string but never cropped
# errors: 1, 2, 3, 4
```

## array tools

### bb_array_is_empty

```
bb_array_is_empty <array-vn>
# check if array is empty
# errors: 2, 3, 7, 91 (not empty)
```

### bb_array_length

```
bb_array_length <ret-vn> <array-vn>
# return length of array
# errors: 2, 3, 7, 91 (value is no array)
```

### bb_array_contains_value

```
bb_array_contains_value <array-vn> <string>
# check if array contains value <string>, return 0 if found
# errors: 2, 3, 10
```

### bb_array_contains_key

```
bb_array_contains_key <array-vn> <key>
# check if array contains key
# errors: 2, 3, 10
```

### bb_array_compare

```
bb_array_compare <array1-vn> <array2-vn>
# deep compare two arrays, return 0 if identically
# errors: 2, 3, 91 (not identically)
```

### bb_array_add

```
bb_array_add <array-vn> <element> [..]
# add elements to existing array
# errors: 2, 3
```

### bb_array_concat

```
bb_array_concat <array-vn> <addon-array-vn>
# add contents of addon-array to to existing array
# indices of addon-array will be lost, the next free indices of array are used
# errors: 2, 3
```

### bb_array_set

```
bb_array_set <array-vn> [<element> [..]]
# overwrite contents of array
# for indirect version see bb_array_copy
# errors: 2, 3
```

### bb_array_insert

```
bb_array_insert <array-vn> <key> <element>
# insert an element at specified key
# elements in way will recursivly be copied to key+1
# note: in sparse arrays not all keys > key might be touched
# errors: 2, 3
```

### bb_array_get

```
bb_array_get <ret-vn> <array-vn> <key>
# returns element of array
# errors: 2, 3, 10
```

### bb_array_clear

```
bb_array_clear <array_vn>
# clear a already defined array
# errors: 2, 3
```

### bb_array_copy_without_vals

```
bb_array_copy_without_vals <dst-vn> <src-vn> <val> [..]
# copies array named srv-vn into array called dst-vn while skipping all
# specified vals
# errors: 2
```

### bb_array_remove_vals

```
bb_array_remove_vals <array-vn> <val> [..]
# removes specified vals from array named array-vn
# errors: 2, 3
```

### bb_array_unsparse

```
bb_array_unsparse <ret-array-vn> <array-vn>
# returns non sparse version of input array, filled with by empty values
# filling will be done from index 0 to max index in source array
# e.g. ([1]=1 [3]=3 [6]=6) would be converted to ("" 1 "" 3 "" "" 6)
# errors: 2, 3, 7, 11
```

### bb_array_compact

```
bb_array_compact <ret-array-vn> <array-vn>
# returns compacted array, all sparse areas will be removed
# e.g. ([0]=0 [2]=2) would be converted to ([0]=0 [1]=2)
# erors: 2, 3, 7, 11
```

### bb_array_is_sparse

```
bb_array_is_sparse <array-vn>
# checks if array is sparse
# errors: 1, 2, 3, 7, 11, 91 (array is not sparse)
```

### bb_array_reverse

```
bb_array_reverse <ret-array-vn> <array-vn>
# returns reversed array
# same indices are used, only data is swapped (0 with n, 1 with (n-1), ..)
# erors: 2, 3
```

### bb_array_iter

```
bb_array_iter <array-vn> <iterfun-name> [<xarg> [..]]
# iter-fun must take: <element> [<xarg> [..]]
# errors: 2, 3, 8, 91 (one or more iterfun calls failed)
```

### bb_array_iter_i

```
bb_array_iter_i <array-vn> <iterfun-name> [<xarg> [..]]
# iter-fun must take: <element> <index> [<xarg> [..]]
# errors: 2, 3, 8, 91 (one or more iterfun calls failed)
```

### bb_array_iter_k

```
bb_array_iter_k <array-vn> <iterfun-name> [<xarg> [..]]
# iter-fun must take: <element> <key> [<xarg> [..]]
# errors: 2, 3, 8, 91 (one or more iterfun calls failed)
```

### bb_array_map

```
bb_array_map <ret-array-vn> <array-vn> <map-fun> [<xarg> [..]]
# map-fun must take: <ret-vn> <src-value> [<xarg> [..]]
# errors: 2, 3, 8, 91 (one or more mapfun calls failed)
```

### bb_array_map_i

```
bb_array_map_i <ret-array-vn> <array-vn> <map-fun> [<xarg> [..]]
# map-fun must take: <ret-vn> <src-value> <index> [<xarg> [..]]
# errors: 2, 3, 8, 91 (one or more mapfun calls failed)
```

### bb_array_map_k

```
bb_array_map_k <ret-array-vn> <array-vn> <map-fun> [<xarg> [..]]
# map-fun must take: <ret-vn> <src-value> <key> [<xarg> [..]]
# errors: 2, 3, 8, 91 (one or more mapfun calls failed)
```

### bb_array_map2

```
bb_array_map2 <ret-array-vn> <array1-vn> <array2-vn> <map-fun> 
#                [<xarg> [..]]
# map-fun must take: <ret-vn> <value1> <value2> <key1> <key2> [<xarg> [..]]
# note: this function processes elements one by one and ignores keys
# errors: 2, 3, 8, 91 (src1 and src2 have different length),
#         92 (one or more mapfun calls failed)
```

### bb_array_map2_i

```
bb_array_map2_i <ret-array-vn> <array1-vn> <array2-vn> <map-fun> 
#                  [<xarg> [..]]
# map-fun must take: <ret-vn> <value1> <value2> <key1> <key2> [<xarg> [..]]
# note: this function processes elements one by one and ignores keys
# errors: 2, 3, 8, 91 (src1 and src2 have different length),
#         92 (one or more mapfun calls failed)
```

### bb_array_map2_k

```
bb_array_map2_k <ret-array-vn> <array1-vn> <array2-vn> <map-fun> 
#                  [<xarg> [..]]
# map-fun must take: <ret-vn> <value1> <value2> <key1> <key2> [<xarg> [..]]
# note: this function processes elements one by one and ignores keys
# errors: 2, 3, 8, 91 (src1 and src2 have different length),
#         92 (one or more mapfun calls failed)
```

### bb_array_fold_right

```
bb_array_fold_right <ret-array-vn> <array-vn> <fold-fun> [<xarg> [..]]
# folds an array from right to left
# fold-fun must take: <ret-vn> <element> <prev-result> [<xarg> [..]]
# errors: 2, 3, 8, 91 (fold-fun call failed)
```

### bb_array_fold_right_i

```
bb_array_fold_right_i <ret-array-vn> <array-vn> <fold-fun> [<xarg> [..]]
# folds an array from right to left
# fold-fun must take: <ret-vn> <element> <prev-result> <index> [<xarg> [..]]
# errors: 2, 3, 8, 91 (fold-fun call failed)
```

### bb_array_fold_right_k

```
bb_array_fold_right_k <ret-array-vn> <array-vn> <fold-fun> [<xarg> [..]]
# folds an array from right to left
# fold-fun must take: <ret-vn> <element> <prev-result> <index> [<xarg> [..]]
# errors: 2, 3, 8, 91 (fold-fun call failed)
```

### bb_array_fold_left

```
bb_array_fold_left <ret-array-vn> <array-vn> <fold-fun> [<arg> [..]]
# folds an array from left to right
# fold-fun must take: <ret-vn> <element> <prev-result> [<xarg> [..]]
# errors: 2, 3, 8, 91 (fold-fun call failed)
```

### bb_array_fold_left_i

```
bb_array_fold_left_i <ret-array-vn> <array-vn> <fold-fun> [<arg> [..]]
# folds an array from left to right
# fold-fun must take: <ret-vn> <element> <prev-result> [<xarg> [..]]
# errors: 2, 3, 8, 91 (fold-fun call failed)
```

### bb_array_fold_left_k

```
bb_array_fold_left_k <ret-array-vn> <array-vn> <fold-fun> [<arg> [..]]
# folds an array from left to right
# fold-fun must take: <ret-vn> <element> <prev-result> [<xarg> [..]]
# errors: 2, 3, 8, 91 (fold-fun call failed)
```

### bb_array_filter

```
bb_array_filter <ret-array-vn> <array-vn> <filter-fun> [<xarg> [..]]
# filterfun must take: <element> [<xarg> [..]]
# and shall return 0 if the element is to be kept in output array
# errors: 2, 3, 8
```

### bb_array_filter_i

```
bb_array_filter_i <ret-array-vn> <array-vn> <filter-fun> [<xarg> [..]]
# filterfun must take: <element> <index> [<xarg> [..]]
# and shall return 0 if the element is to be kept in output array
# errors: 2, 3, 8
```

### bb_array_filter_k

```
bb_array_filter_k <ret-array-vn> <array-vn> <filter-fun> [<xarg> [..]]
# filterfun must take: <element> <key> [<xarg> [..]]
# and shall return 0 if the element is to be kept in output array
# errors: 2, 3, 8
```

### bb_array_copy

```
bb_array_copy <ret-dst-array-vn> <src-array-vn>
# copys an array, target array must exist and if non empty will be merged
# errors: same as bb_array_map
```

### bb_array_unique

```
bb_array_unique <ret-dst-array-vn> <src-array-vn>
# copies array but skip non unique elements
# errors: 2, 3
```

### bb_array_length

```
bb_array_length <ret-vn> <array-vn>
# returns the length of an array
# errors: 2, 3
```

### bb_mapfun_identity

```
bb_mapfun_identity <ret-vn> <value>
# the identity fun
# errors: 2, 3
```

### bb_array_join_by

```
bb_array_join_by <ret-vn> <array-vn> <join-string>
# joins array element with join-string, returns "" if array is empty
# errors: 1, 2, 3, 7, 11
```

### bb_array_join_by_direct

```
bb_array_join_by_direct <ret-vn> <join-string> [<element> [..]]
# joins array element with join-string, returns "" if no elements specified
# errors: 1, 2, 3, 7, 11
```

### bb_array_of_string

```
bb_array_of_string <ret-array-vn> <string> [ <batch-size> ]
# converts a string into an array of strings with size batch-size and copy
# it into the array called dst-vn
# the default batch-size is 1
# note: the last batch may be shorter than batch-size
# errors: 2
```

## dict (associative array) tools

### bb_dict_keys

```
bb_dict_keys <ret-array-vn> <dict-vn>
# stores keys of dict to ret-keys-array
# errors: 2, 3
```

## hexadecimal

### bb_hex_validate

```
bb_hex_validate <caller-name> <hex-string> [..]
# validates one or more hex strings
# valid variable names match the following regex: ^[[:xdigit:]]*$
# errors: 2, 91 (illegal hex code)
```

### bb_hex_read

```
bb_hex_read <ret-vn>
# converts binary data read from stdin to its hex representation
# make sure it does not run in subshell, call it e.g.: bb_hex_read x < <(date)
# requires: /usr/bin/od
# errors: 2, 3, 91 (od not found)
```

### bb_bes_of_hex

```
bb_bes_of_hex <ret-vn> <hex>
# converts hex to backslash escape sequence, which can be converted to data
# by passing to: printf "%b"
# hex is interpreted as stream of bytes (means it must be of even length)
# errors: 2, 91 (hex is of odd length)
```

### bb_hex_write

```
bb_hex_write <hex>
# writes hex encoded data as binary data to stdout
# hex is interpreted as stream of bytes (means it must be of even length)
# errors: 2, 91 (hex is of odd length), 92 (invalid hex char)
```

### bb_hex_xor

```
bb_hex_xor <ret-vn> <hex1> <hex2>
# returns hex1 xor hex2 as hex
# errors: 2, 3, 91 (input length vary), 92 (invaid hex char)
```

## lock files

## error handling: handle internal errors in a configurable way

### bb_ierr

```
bb_ierr <msg> [..]
# call this on internal error, depending on mode it does:
#  mode "fatal": pass msg to bb_fatal
#  other mode: print error msg with specified loglevel
```

### bb_ierr_set_mode

```
bb_ierr_set_mode [fatal|<msg-loglevel>]
# default is "err"
```

### bb_fatal

```
bb_fatal <message> [..]
# end program execution after logging message prepended with 'fatal error'
# since it produces some lines of output it is meant for internal errors
# only, if you want to abort your script with a message use bb_quit
```

## msg: log/show messages

### bb_msg_levelname_of_string

```
bb_msg_levelname_of_string <ret-vn> <string>
# returns level as string if input was integer
# else returns string if it is a valid level
# errors: 2, 3, 4
```

### bb_msg_int_of_level

```
bb_msg_int_of_level <ret-vn> <level>
# convert level name to int or return int unmodified if in valid range
# errors: 2, 3, 4
```

### bb_msg_set_target

```
bb_msg_set_target <target-name> <type>[:<arg>] <min-level>[:<max-level]
#                   <prefix> [[no]trace]
# where type can be one of: stdout, stderr, file, syslog
# and arg is filename for file, facility for syslog
# level is one of: debug[123]?, info, notice, warning, err, crit, alert, emerg
# prefix can be: none, timestamp, prog, progpid, full
# if notrace is specified, no stack traces will be print to that target
# notrace is the default
# note: target-name must match regex ^[[:alpha:]][[:alnum:]_]*$
# errors: 2, 4
```

### bb_msg_del_target

```
bb_msg_del_target <target-name>
# errors: 2
```

### bb_msg_clear_targets

```
bb_msg_clear_targets
# remove all message targets
# errors: 2
# shellcheck disable=SC2120
```

### bb_msg

```
bb_msg <level> <string> [..]
# logs messages to configured targets
# note: will return 0, even if level is illegal
# errors: 2, 4
```

### bb_msg_iterator

```
bb_msg_iterator <msg> <level>
# iterator to output msgs in iterations
# errors: like bb_msg
```

### bb_msg_stacktrace

```
bb_msg_stacktrace <level> [<skip-count>]
# outputs stack trace
# skips skip-count elements of stack, default is 2
# errors: 1, 2
```

### bb_msg_from_stdin

```
bb_msg_from_stdin <level> <prefix>
# reads from fd 0 until EOF and calls bb_msg for each line read
# every line is prefixed with prefix
# will not do anything if fd 0 is connected to a terminal
# note: will stop if bb_msg fails and forward the return value
# errors: 2, 4
```

### bb_msgf

```
bb_msgf <level> <format> [<format-arg> [..]]
# logs printf style messages to configured targets
# note: will return 0, even if level is illegal
# errors: 2, 4
```

### bb_msg_stacktrace

```
bb_msg_stacktrace <ret-array-vn> [<skip>]
# create a human readable stacktrace, excluding this function and
# returns it as array of strings
# optional arg skip can be used to manually configure how many inner
# levels will not be displayed
# errors: 2, 3
```

## log functions (deprecated, use msg instead)

### bb_logf

```
bb_logf <message> [..]
# logs message file name in $LOG_FILE if non zero
```

### bb_log

```
bb_log <message> [..]
# logs message to stderr and if define to file name in $LOG_FILE
```

### bb_log_debug

```
bb_log_debug <level> <message> [..]
# logs message if $LOG_LEVEL is set and it is level <= $LOG_LEVEL
# errors: 2
```

## flow control and error handling

### bb_finalize

```
bb_finalize
# unregisters all traps
```

### bb_quit

```
bb_quit [<exit-code> [<log message>] [..]]
# the safe way to clean exit a script using bash_base
```

### bb_exec

```
bb_exec <cmd> [arg [..]]
# execs cmd with args if specified
```

### bb_traps_setup

```
bb_traps_setup
# will be removed in the future
```

### bb_traps_disable

```
bb_traps_disable
# will be removed in the future
```

### bb_version

```
bb_version <ret-vn>
# returns the version of bash base
# errors: 2, 3
```

### bb_register_exit_hook

```
bb_register_exit_hook [ <hook-name> ] <function-name>
# registers a function to be called on exit, it is not possible to
# pass arguments to that function
# hooks will be called sorted by hooknames
# note: the default hook-name is the function-name
```

### bb_unregister_exit_hook

```
bb_unregister_exit_hook <hookn-ame>
# removes a registered exit hook
# errors: 2
```

### bb_load_lib

```
bb_load_lib <name>
# sources file for lib specified
# if name is no absolute, relative or local path it tries:
#   /usr/libexec/<name>.bash
#   /usr/libexec/<name>/<name>.bash
# absolute paths start with /
# local paths start with ./
# relative paths start with ../
# errors: 2, 5, 8, 91 (lib not found)
```

### bb_load_libs

```
bb_load_libs <name> [..]
# load one or more libs but continue if some fail to load
# errors: 2, 91 (not all libs loaded)
```

### bb_require_libs

```
bb_require_libs <name> [..]
# load one or more libs and raise fatal error if any fail
# errors: 2, bb_fatal on any other error
```
