#!/bin/bash

# This code block is a hack to enable execution of demo scripts without
# global installation of bash-base. Do not use it, in your production scripts.
BBLIBFILE=bash-base.bash
for bbloc in bash-base.bash ../libexec/bash-base.bash \
    /usr/libexec/bash-base.bash
do
    if [ -f "$bbloc" ]
    then
	BBLIBFILE="$bbloc"
	break
    fi
done
source "$BBLIBFILE" || {
    echo "${0##*/}: fatal error: failed to source bash-base.bash" >&2
    exit 1
}

function cleanup() {
    bb_msg info "cleanup called"
}

# replace the contents with your code
function main() {
    # (un)comment lines below to test what happens on various failures

    bb_msg info "main() was called with args: '$*'"

    # make sure cleanup() is called at script exit
    bb_register_exit_hook cleanup

    # bb_fatal aborts execution and outputs message with level emerg
    # by default emerg messages will show a stack trace as well
    bb_fatal 'kenny?'

    # on undefined variable used a message with level emerg will be logged
    # shellcheck disable=SC2154
    echo "$NoSuchVariableDefined"

    # commands that fail will abort execution as well
    false

    # this will sucessfully end function main, but since the return value
    # is != 0 it will trigger an error where main was called
    return 123
}

main "$@"

# always use bb_quit to quit, it cares about traps and hooks
bb_quit
